package com.example.flappybird;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Rect;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.view.View;
import android.widget.ImageView;
import android.os.Handler;
import android.view.MotionEvent;
import android.widget.TextView;
import androidx.appcompat.app.AppCompatActivity;
import java.util.ArrayList;
import java.util.List;


public class MainActivity extends AppCompatActivity {

    // Sprites
    ImageView PlayButton;
    ImageView Logo;
    ImageView Bird;
    ImageView GameOver;
    ImageView EndMenu;

    // Texts
    TextView counterTextView;
    TextView endScoreTextView;
    TextView hScoreTextView;

    // Sounds
    MediaPlayer pointSound;
    MediaPlayer flapSound;
    MediaPlayer gameOverSound;

    // Key for prefs
    private static final String PREFS_NAME = "FlappyBirdPrefs";

    // Bird variables
    float birdY; // Bird vertical position
    float birdYInit; // Bird initial vertical position
    float birdV = 0; // Bird vertical velocity
    float g = 2; // Gravity
    float flap = -23; // Flap strength

    // Game running variables
    boolean gameRunning = false; // Is  the game running ?
    private final Handler handler = new Handler();
    private final int updateInterval = 30;

    // Score counter variables
    int score;
    int highScore;

    // Pipes variables
    List<Pipe> pipes = new ArrayList<>();
    int pipeCreationInterval = 3000;
    boolean pipeGenerationRunning = false;


    @SuppressLint("ClickableViewAccessibility")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // References to layout elements
        PlayButton = findViewById(R.id.playbutton);
        Logo = findViewById(R.id.logo);
        Bird = findViewById(R.id.bird);
        GameOver = findViewById(R.id.gameOver);
        EndMenu = findViewById(R.id.endMenu);
        counterTextView = findViewById(R.id.counter);
        endScoreTextView = findViewById(R.id.endScore);
        hScoreTextView = findViewById(R.id.hScore);
        birdY = Bird.getY(); //get initial bird position
        birdYInit = birdY;
        pointSound = MediaPlayer.create(MainActivity.this, R.raw.point);
        flapSound = MediaPlayer.create(MainActivity.this, R.raw.wing);
        gameOverSound = MediaPlayer.create(MainActivity.this, R.raw.gameovermario);

        // Flap
        findViewById(R.id.main).setOnTouchListener((v, event) -> {
            if (gameRunning && event.getAction() == MotionEvent.ACTION_DOWN) {
                birdV = flap;
                flapSound.start(); // Play Flap sound
            }
            return true;
        });

    }

    // Method to start the game
    public void startGame(View view) {

        // Hide assets
        PlayButton.setVisibility(View.GONE);
        Logo.setVisibility(View.GONE);
        GameOver.setVisibility(View.GONE);
        EndMenu.setVisibility(View.GONE);

        // Stop "Game Over" Sound
        if (gameOverSound.isPlaying()) {
            gameOverSound.stop();
            try {
                gameOverSound.prepare();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        // Remove pipes
        for (Pipe pipe : pipes) {
            pipe.remove();
        }
        pipes.clear(); // clear pipes

        // Start game
        pipeGenerationRunning = false;
        birdY = birdYInit;
        score = 0;
        endScoreTextView.setText(""); // Empty end score text view
        hScoreTextView.setText(""); // Empty end score text view
        counterTextView.setText("0"); // Start counter at 0
        gameRunning = true;
        startGameLoop();
        startPipeGeneration();

    }

    // Method to end the game
    public void endGame() {

        // Update High score
        highScore = getHighScore();
        if (score > highScore) {
            saveHighScore(score);
            highScore = score;
        }

        birdV = 0; // Stop movement
        gameRunning = false; // stop game
        PlayButton.setVisibility(View.VISIBLE); // Show play Button
        GameOver.setVisibility(View.VISIBLE); // Show play Game Over
        EndMenu.setVisibility(View.VISIBLE); // Show play End Menu
        GameOver.bringToFront();
        EndMenu.bringToFront();
        PlayButton.bringToFront();
        counterTextView.setText(""); // Start counter at ""
        endScoreTextView.setText(String.valueOf(score));  // Set end score  TextView to counter value
        hScoreTextView.setText(String.valueOf(highScore));  // Set h score  TextView to counter value
        endScoreTextView.bringToFront();
        hScoreTextView.bringToFront();
        pipeGenerationRunning = false;
        gameOverSound.start(); // Play game over sound

        handler.removeCallbacksAndMessages(null);

    }

    // Method to check if the game needs to end
    public void checkEndGame() {

        // Verify ground collision
        if (birdY >= findViewById(R.id.main).getHeight() - Bird.getHeight()) {
            endGame();
        }

        // Verify pipes collision
        for (Pipe pipe : pipes) {
            Rect birdRect = new Rect(
                    (int) Bird.getX(),
                    (int) Bird.getY(),
                    (int) (Bird.getX() + Bird.getWidth()),
                    (int) (Bird.getY() + Bird.getHeight())
            );

            Rect topPipeRect = new Rect(
                    (int) pipe.topPipe.getX(),
                    (int) pipe.topPipe.getY(),
                    (int) (pipe.topPipe.getX() + pipe.topPipe.getWidth()),
                    (int) (pipe.topPipe.getY() + pipe.topPipe.getHeight())
            );

            Rect bottomPipeRect = new Rect(

                    (int) pipe.bottomPipe.getX(),
                    (int) pipe.bottomPipe.getY(),
                    (int) (pipe.bottomPipe.getX() + pipe.bottomPipe.getWidth()),
                    (int) (pipe.bottomPipe.getY() + pipe.bottomPipe.getHeight())
            );

            if (Rect.intersects(birdRect, topPipeRect) || Rect.intersects(birdRect, bottomPipeRect)) {
                endGame();
            }
        }


    }

    // Method to create pipes
    private void createPipe() {
        View mainView = findViewById(R.id.main);
        float screenHeight = mainView.getHeight();
        float screenWidth = mainView.getWidth();
        Pipe newPipe = new Pipe(this, screenHeight, screenWidth);
        pipes.add(newPipe);
        counterTextView.bringToFront(); // Makes sure that counter is always on front
    }

    // Method to start the game on a loop
    public void startGameLoop() {
        handler.postDelayed(new Runnable(){
            @Override
            public void run() {
                if (gameRunning){

                    // Check if game ended
                    checkEndGame();

                    // Update Bird variable status
                    birdV += g; // Apply gravity on bird velocity variable
                    birdY += birdV; // Update bird position variable

                    // Collision with screen borders
                    if (birdY < 0) {
                        birdY = 0;
                        birdV = 0;
                    }

                    // Apply variables on bird sprite
                    Bird.setY(birdY);


                    // Update Pipes variables
                    for (int i = 0; i < pipes.size(); i++) {
                        Pipe pipe = pipes.get(i);
                        pipe.move();

                        // Remove pipe outside the screen
                        if (pipe.pipeX + pipe.PIPE_WIDTH < 0) {
                            pipe.remove();
                            pipes.remove(i); // Remove pipe from list
                            i--;
                        }
                    }

                    // Check if it scored
                    checkForScore();

                    // Repeat Loop
                    handler.postDelayed(this, updateInterval);
                }
            }
        },updateInterval);
   }

    // Method to generate pipes periodically
    public void startPipeGeneration() {
        if (!pipeGenerationRunning) {
            pipeGenerationRunning = true;
            handler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    if (gameRunning) {
                        createPipe();
                        handler.postDelayed(this, pipeCreationInterval);
                    }
                }
            }, pipeCreationInterval);
        }
    }

    // Method for score counter
    public void checkForScore() {
        for (Pipe pipe : pipes) {
            if (!pipe.isScored && Bird.getX() > pipe.pipeX + pipe.PIPE_WIDTH) {
                score++;  // Increment o score
                pipe.isScored = true;
                counterTextView.setText(String.valueOf(score));  // Update score counter TextView
                pointSound.start(); // Play point sound
            }
        }
    }

    // Method to get high score
    private int getHighScore() {
        SharedPreferences preferences = getSharedPreferences(PREFS_NAME, MODE_PRIVATE);
        return preferences.getInt("HighScore", 0);
    }

    // Methode for saving high score
    private void saveHighScore(int newScore) {
        SharedPreferences preferences = getSharedPreferences(PREFS_NAME, MODE_PRIVATE);
        SharedPreferences.Editor editor = preferences.edit();
        editor.putInt("HighScore", newScore);
        editor.apply();
    }



}